@@ -11,9 +11,6 @@
  */
 
 #include "config.h"
-#if HAVE_SECURE_GETENV
-#define _GNU_SOURCE
-#endif
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
